@@ -1,7 +1,9 @@
 package ldap
 
 import (
+	"context"
 	"crypto/tls"
+	"log"
 	"testing"
 
 	ber "github.com/go-asn1-ber/asn1-ber"
@@ -320,3 +322,91 @@ func Test_addControlDescriptions(t *testing.T) {
 		})
 	}
 }
+
+func TestEscapeDN(t *testing.T) {
+	tests := []struct {
+		name string
+		dn   string
+		want string
+	}{
+		{name: "emptyString", dn: "", want: ""},
+		{name: "comma", dn: "test,user", want: "test\\,user"},
+		{name: "numberSign", dn: "#test#user#", want: "\\#test#user#"},
+		{name: "backslash", dn: "\\test\\user\\", want: "\\\\test\\\\user\\\\"},
+		{name: "whitespaces", dn: "  test user  ", want: "\\  test user \\ "},
+		{name: "nullByte", dn: "\u0000te\x00st\x00user" + string(rune(0)), want: "\\00te\\00st\\00user\\00"},
+		{name: "variousCharacters", dn: "test\"+,;<>\\-_user", want: "test\\\"\\+\\,\\;\\<\\>\\\\-_user"},
+		{name: "multiByteRunes", dn: "test\u0391user ", want: "test\u0391user\\ "},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			if got := EscapeDN(tt.dn); got != tt.want {
+				t.Errorf("EscapeDN(%s) = %s, expected %s", tt.dn, got, tt.want)
+			}
+		})
+	}
+}
+
+func TestSearchAsync(t *testing.T) {
+	l, err := DialURL(ldapServer)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer l.Close()
+
+	searchRequest := NewSearchRequest(
+		baseDN,
+		ScopeWholeSubtree, DerefAlways, 0, 0, false,
+		filter[2],
+		attributes,
+		nil)
+
+	srs := make([]*Entry, 0)
+	ctx := context.Background()
+	r := l.SearchAsync(ctx, searchRequest, 64)
+	for r.Next() {
+		srs = append(srs, r.Entry())
+	}
+	if err := r.Err(); err != nil {
+		log.Fatal(err)
+	}
+
+	t.Logf("TestSearcAsync: %s -> num of entries = %d", searchRequest.Filter, len(srs))
+}
+
+func TestSearchAsyncAndCancel(t *testing.T) {
+	l, err := DialURL(ldapServer)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer l.Close()
+
+	searchRequest := NewSearchRequest(
+		baseDN,
+		ScopeWholeSubtree, DerefAlways, 0, 0, false,
+		filter[2],
+		attributes,
+		nil)
+
+	cancelNum := 10
+	srs := make([]*Entry, 0)
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+	r := l.SearchAsync(ctx, searchRequest, 0)
+	for r.Next() {
+		srs = append(srs, r.Entry())
+		if len(srs) == cancelNum {
+			cancel()
+		}
+	}
+	if err := r.Err(); err != nil {
+		log.Fatal(err)
+	}
+
+	if len(srs) > cancelNum+3 {
+		// the cancellation process is asynchronous,
+		// so it might get some entries after calling cancel()
+		t.Errorf("Got entries %d, expected < %d", len(srs), cancelNum+3)
+	}
+	t.Logf("TestSearchAsyncAndCancel: %s -> num of entries = %d", searchRequest.Filter, len(srs))
+}
